﻿using System.Collections.Generic;
using System.Linq;

namespace jello
{
    class InternalState
    {
        private short _Register = 0;
        private Dictionary<short, short> _Memory = new Dictionary<short, short> { };
        private short _Input = 0;
        private short _Output = 0;
        private List<string> _CodeLines = new List<string> { };
        private int _Pointer = 0;
        private int _LastPointer = 0;

        override public string ToString()
        {
            return "InternalState";
        }

        public InternalState()
        {
        }

        public InternalState(short register, Dictionary<short, short> memory, short input, short output, int pointer, List<string> codelines)
        {
            _Register = register;
            _Memory = memory;
            _Input = input;
            _Output = output;
            _CodeLines = codelines;
            _Pointer = pointer;
        }

        public string Dump()
        {
            string dump = "";
            bool first = true;
            dump += "   @REG = " + _Register.ToString() + "\n";
            dump += "   @MEM = {";
            foreach (KeyValuePair<short, short> item in _Memory)
            {
                if (!first) dump += ", ";
                else first = false;
                dump += item.Key.ToString() + ":" + item.Value.ToString();
            }
            dump += "}\n";
            dump += "   @INP = " + _Input.ToString() + "\n";
            dump += "   @OUT = " + _Output.ToString();
            return dump;
        }

        public bool CleanUp()
        {
            if (_Register > 255 || _Register < 0)
                return true;
            if (_Input > 255 || _Input < 0)
                return true;
            if (_Output > 255 || _Output < 0)
                return true;
            foreach (KeyValuePair<short, short> item in _Memory)
            {
                if (item.Value > 255 || item.Value < 0 || item.Key > 255 || item.Key < 0)
                    return true;
            }
            return false;
        }

        public short Register
        {
            get { return _Register; }
            set { _Register = value; }
        }

        public short Input
        {
            get { return _Input; }
            set { _Input = value; }
        }

        public short Output
        {
            get { return _Output; }
            set { _Output = value; }
        }

        public int Pointer
        {
            get { return _Pointer; }
            set {
                _LastPointer = _Pointer;
                _Pointer = value;
            }
        }

        public int LastPointer
        {
            get { return _LastPointer; }
        }

        public List<string> CodeLines
        {
            get { return _CodeLines; }
            set { _CodeLines = value; }
        }

        public short GetMemory(short index)
        {
            if (_Memory.Keys.Contains(index))
                return _Memory[index];
            else
                return 0;
        }

        public void SetMemory(short index, short value)
        {
            _Memory[index] = value;
        }

        //TODO: Add stack of 16 items, this will allow for a GOSUB type command and stack manipulation.
    }
}
